# coding: utf-8

"""
    mod-dlm API

    mod-dlm is an open source domain specific language to describe prefab construction elements. This project provides tools for defining entity models and automatically generating CRUD APIs, enabling suppliers to create and manage catalogs of their products easily.

    The version of the OpenAPI document: 1.0.12
    Contact: info@mod.construction
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json




from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class PrefabElementSustainability(BaseModel):
    """
    Environmental and sustainability performance such as EPDs or recyclability.
    """ # noqa: E501
    country_of_manufacturing: StrictStr = Field(description="Country where the element is manufactured", alias="countryOfManufacturing")
    classification: StrictStr = Field(description="Simplified sustainability score, where A+ indicates highest performance. Define based on EPD or lifecycle impact benchmarks.")
    voc_emissions: Optional[StrictStr] = Field(default=None, description="Volatile Organic Compounds emissions level (None, Low, Medium, High)", alias="VOCEmissions")
    recyclability: Optional[Union[Annotated[float, Field(le=100, strict=True, ge=0)], Annotated[int, Field(le=100, strict=True, ge=0)]]] = Field(default=None, description="Percentage of the element that is recyclable, from 0 to 100%")
    energy_efficiency: StrictStr = Field(description="Energy performance classification, combining insulation tiers and certifications (LEED, BREEAM, etc.).", alias="energyEfficiency")
    __properties: ClassVar[List[str]] = ["countryOfManufacturing", "classification", "VOCEmissions", "recyclability", "energyEfficiency"]

    @field_validator('classification')
    def classification_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('A+', 'A', 'B', 'C', 'D',):
            raise ValueError("must be one of enum values ('A+', 'A', 'B', 'C', 'D')")
        return value

    @field_validator('voc_emissions')
    def voc_emissions_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('None (<0.01 mg/m³)', 'Very Low (<0.1 mg/m³)', 'Low (<0.3 mg/m³)', 'Moderate (0.3–1.0 mg/m³)', 'High (>1.0 mg/m³)',):
            raise ValueError("must be one of enum values ('None (<0.01 mg/m³)', 'Very Low (<0.1 mg/m³)', 'Low (<0.3 mg/m³)', 'Moderate (0.3–1.0 mg/m³)', 'High (>1.0 mg/m³)')")
        return value

    @field_validator('energy_efficiency')
    def energy_efficiency_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('R-1', 'R-2', 'R-5', 'Energy Star', 'LEED Certified', 'LEED Silver', 'LEED Gold', 'LEED Platinum', 'BREEAM Good', 'BREEAM Very Good', 'BREEAM Excellent', 'BREEAM Outstanding', 'Passive House', 'DGNB Gold',):
            raise ValueError("must be one of enum values ('R-1', 'R-2', 'R-5', 'Energy Star', 'LEED Certified', 'LEED Silver', 'LEED Gold', 'LEED Platinum', 'BREEAM Good', 'BREEAM Very Good', 'BREEAM Excellent', 'BREEAM Outstanding', 'Passive House', 'DGNB Gold')")
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PrefabElementSustainability from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
            },
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of PrefabElementSustainability from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "countryOfManufacturing": obj.get("countryOfManufacturing"),
            "classification": obj.get("classification"),
            "VOCEmissions": obj.get("VOCEmissions"),
            "recyclability": obj.get("recyclability"),
            "energyEfficiency": obj.get("energyEfficiency")
        })
        return _obj


